@@ -12,3 +12,13 @@ __version__ = open(__cqlengine_version_path__, 'r').readline().strip()
 SizeTieredCompactionStrategy = "SizeTieredCompactionStrategy"
 LeveledCompactionStrategy = "LeveledCompactionStrategy"
 
+
+ANY = "ANY"
+ONE = "ONE"
+TWO = "TWO"
+THREE = "THREE"
+QUORUM = "QUORUM"
+LOCAL_QUORUM = "LOCAL_QUORUM"
+EACH_QUORUM = "EACH_QUORUM"
+ALL = "ALL"
+
