@@ -198,6 +198,10 @@ class Column(object):
     def get_cql(self):
         return '"{}"'.format(self.db_field_name)
 
+    def _val_is_null(self, val):
+        """ determines if the given value equates to a null value for the given column type """
+        return val is None
+
 
 class Bytes(Column):
     db_type = 'blob'
@@ -252,6 +256,10 @@ class Integer(Column):
         return self.validate(value)
 
 
+class BigInt(Integer):
+    db_type = 'bigint'
+
+
 class VarInt(Column):
     db_type = 'varint'
 
@@ -328,11 +336,9 @@ class DateTime(Column):
             else:
                 raise ValidationError("'{}' is not a datetime object".format(value))
         epoch = datetime(1970, 1, 1, tzinfo=value.tzinfo)
-        offset = 0
-        if epoch.tzinfo:
-            offset_delta = epoch.tzinfo.utcoffset(epoch)
-            offset = offset_delta.days*24*3600 + offset_delta.seconds
-        return long(((value  - epoch).total_seconds() - offset) * 1000)
+        offset = epoch.tzinfo.utcoffset(epoch).total_seconds() if epoch.tzinfo else 0
+
+        return long(((value - epoch).total_seconds() - offset) * 1000)
 
 
 class Date(Column):
@@ -402,12 +408,7 @@ class TimeUUID(UUID):
         global _last_timestamp
 
         epoch = datetime(1970, 1, 1, tzinfo=dt.tzinfo)
-
-        offset = 0
-        if epoch.tzinfo:
-            offset_delta = epoch.tzinfo.utcoffset(epoch)
-            offset = offset_delta.days*24*3600 + offset_delta.seconds
-
+        offset = epoch.tzinfo.utcoffset(epoch).total_seconds() if epoch.tzinfo else 0
         timestamp = (dt  - epoch).total_seconds() - offset
 
         node = None
@@ -526,6 +527,15 @@ class BaseContainerColumn(Column):
         """
         raise NotImplementedError
 
+    def _val_is_null(self, val):
+        return not val
+
+
+class BaseContainerQuoter(ValueQuoter):
+
+    def __nonzero__(self):
+        return bool(self.value)
+
 
 class Set(BaseContainerColumn):
     """
@@ -535,7 +545,7 @@ class Set(BaseContainerColumn):
     """
     db_type = 'set<{}>'
 
-    class Quoter(ValueQuoter):
+    class Quoter(BaseContainerQuoter):
 
         def __str__(self):
             cq = cql_quote
@@ -625,12 +635,15 @@ class List(BaseContainerColumn):
     """
     db_type = 'list<{}>'
 
-    class Quoter(ValueQuoter):
+    class Quoter(BaseContainerQuoter):
 
         def __str__(self):
             cq = cql_quote
             return '[' + ', '.join([cq(v) for v in self.value]) + ']'
 
+        def __nonzero__(self):
+            return bool(self.value)
+
     def __init__(self, value_type, default=set, **kwargs):
         return super(List, self).__init__(value_type=value_type, default=default, **kwargs)
 
@@ -736,7 +749,7 @@ class Map(BaseContainerColumn):
 
     db_type = 'map<{}, {}>'
 
-    class Quoter(ValueQuoter):
+    class Quoter(BaseContainerQuoter):
 
         def __str__(self):
             cq = cql_quote
