@@ -9,7 +9,7 @@ from cqlengine.columns import Counter
 
 from cqlengine.connection import connection_manager, execute, RowResult
 
-from cqlengine.exceptions import CQLEngineException
+from cqlengine.exceptions import CQLEngineException, ValidationError
 from cqlengine.functions import QueryValue, Token
 
 #CQL 3 reference:
@@ -22,6 +22,7 @@ class MultipleObjectsReturned(QueryException): pass
 
 class QueryOperatorException(QueryException): pass
 
+
 class QueryOperator(object):
     # The symbol that identifies this operator in filter kwargs
     # ie: colname__<symbol>
@@ -116,10 +117,12 @@ class QueryOperator(object):
     def __hash__(self):
         return hash(self.column.db_field_name) ^ hash(self.value)
 
+
 class EqualsOperator(QueryOperator):
     symbol = 'EQ'
     cql_symbol = '='
 
+
 class IterableQueryValue(QueryValue):
     def __init__(self, value):
         try:
@@ -133,28 +136,34 @@ class IterableQueryValue(QueryValue):
     def get_cql(self):
         return '({})'.format(', '.join(':{}'.format(i) for i in self.identifier))
 
+
 class InOperator(EqualsOperator):
     symbol = 'IN'
     cql_symbol = 'IN'
 
     QUERY_VALUE_WRAPPER = IterableQueryValue
 
+
 class GreaterThanOperator(QueryOperator):
     symbol = "GT"
     cql_symbol = '>'
 
+
 class GreaterThanOrEqualOperator(QueryOperator):
     symbol = "GTE"
     cql_symbol = '>='
 
+
 class LessThanOperator(QueryOperator):
     symbol = "LT"
     cql_symbol = '<'
 
+
 class LessThanOrEqualOperator(QueryOperator):
     symbol = "LTE"
     cql_symbol = '<='
 
+
 class AbstractQueryableColumn(object):
     """
     exposes cql query operators through pythons
@@ -192,23 +201,29 @@ class BatchType(object):
     Unlogged    = 'UNLOGGED'
     Counter     = 'COUNTER'
 
+
 class BatchQuery(object):
     """
     Handles the batching of queries
 
     http://www.datastax.com/docs/1.2/cql_cli/cql/BATCH
     """
+    _consistency = None
 
-    def __init__(self, batch_type=None, timestamp=None):
+    def __init__(self, batch_type=None, timestamp=None, consistency=None):
         self.queries = []
         self.batch_type = batch_type
         if timestamp is not None and not isinstance(timestamp, datetime):
             raise CQLEngineException('timestamp object must be an instance of datetime')
         self.timestamp = timestamp
+        self._consistency = consistency
 
     def add_query(self, query, params):
         self.queries.append((query, params))
 
+    def consistency(self, consistency):
+        self._consistency = consistency
+
     def execute(self):
         if len(self.queries) == 0:
             # Empty batch is a no-op
@@ -228,7 +243,7 @@ class BatchQuery(object):
 
         query_list.append('APPLY BATCH;')
 
-        execute('\n'.join(query_list), parameters)
+        execute('\n'.join(query_list), parameters, self._consistency)
 
         self.queries = []
 
@@ -273,6 +288,8 @@ class AbstractQuerySet(object):
         self._result_idx = None
 
         self._batch = None
+        self._ttl = None
+        self._consistency = None
 
     @property
     def column_family_name(self):
@@ -290,7 +307,7 @@ class AbstractQuerySet(object):
     def __deepcopy__(self, memo):
         clone = self.__class__(self.model)
         for k,v in self.__dict__.items():
-            if k in ['_con', '_cur', '_result_cache', '_result_idx']:
+            if k in ['_con', '_cur', '_result_cache', '_result_idx']: # don't clone these
                 clone.__dict__[k] = None
             elif k == '_batch':
                 # we need to keep the same batch instance across
@@ -351,7 +368,7 @@ class AbstractQuerySet(object):
         if self._batch:
             raise CQLEngineException("Only inserts, updates, and deletes are available in batch mode")
         if self._result_cache is None:
-            columns, self._result_cache = execute(self._select_query(), self._where_values())
+            columns, self._result_cache = execute(self._select_query(), self._where_values(), self._consistency)
             self._construct_result = self._get_result_constructor(columns)
 
     def _fill_result_cache_to_idx(self, idx):
@@ -604,7 +621,7 @@ class AbstractQuerySet(object):
         return self._only_or_defer('defer', fields)
 
     def create(self, **kwargs):
-        return self.model(**kwargs).batch(self._batch).save()
+        return self.model(**kwargs).batch(self._batch).ttl(self._ttl).consistency(self._consistency).save()
 
     #----delete---
     def delete(self, columns=[]):
@@ -630,6 +647,7 @@ class AbstractQuerySet(object):
     def __ne__(self, q):
         return not (self != q)
 
+
 class ResultObject(dict):
     """
     adds attribute access to a dictionary
@@ -641,6 +659,7 @@ class ResultObject(dict):
         except KeyError:
             raise AttributeError
 
+
 class SimpleQuerySet(AbstractQuerySet):
     """
 
@@ -658,6 +677,7 @@ class SimpleQuerySet(AbstractQuerySet):
             return ResultObject(zip(names, values))
         return _construct_instance
 
+
 class ModelQuerySet(AbstractQuerySet):
     """
 
@@ -727,6 +747,11 @@ class ModelQuerySet(AbstractQuerySet):
 
         return column.db_field_name, order_type
 
+    def _get_ttl_statement(self):
+        if not self._ttl:
+            return ""
+        return "USING TTL {}".format(self._ttl)
+
     def values_list(self, *fields, **kwargs):
         """ Instructs the query set to return tuples, not model instance """
         flat = kwargs.pop('flat', False)
@@ -740,6 +765,66 @@ class ModelQuerySet(AbstractQuerySet):
         clone._flat_values_list = flat
         return clone
 
+    def consistency(self, consistency):
+        clone = copy.deepcopy(self)
+        clone._consistency = consistency
+        return clone
+
+    def ttl(self, ttl):
+        clone = copy.deepcopy(self)
+        clone._ttl = ttl
+        return clone
+
+    def update(self, **values):
+        """ Updates the rows in this queryset """
+        if not values:
+            return
+
+        set_statements = []
+        ctx = {}
+        nulled_columns = set()
+        for name, val in values.items():
+            col = self.model._columns.get(name)
+            # check for nonexistant columns
+            if col is None:
+                raise ValidationError("{}.{} has no column named: {}".format(self.__module__, self.model.__name__, name))
+            # check for primary key update attempts
+            if col.is_primary_key:
+                raise ValidationError("Cannot apply update to primary key '{}' for {}.{}".format(name, self.__module__, self.model.__name__))
+
+            val = col.validate(val)
+            if val is None:
+                nulled_columns.add(name)
+                continue
+            # add the update statements
+            if isinstance(col, (BaseContainerColumn, Counter)):
+                val_mgr = self.instance._values[name]
+                set_statements += col.get_update_statement(val, val_mgr.previous_value, ctx)
+
+            else:
+                field_id = uuid4().hex
+                set_statements += ['"{}" = :{}'.format(col.db_field_name, field_id)]
+                ctx[field_id] = val
+
+        if set_statements:
+            ttl_stmt = "USING TTL {}".format(self._ttl) if self._ttl else ""
+            qs = "UPDATE {} SET {} WHERE {} {}".format(
+                self.column_family_name,
+                ', '.join(set_statements),
+                self._where_clause(),
+                ttl_stmt
+            )
+            ctx.update(self._where_values())
+            execute(qs, ctx, self._consistency)
+
+        if nulled_columns:
+            qs = "DELETE {} FROM {} WHERE {}".format(
+                ', '.join(nulled_columns),
+                self.column_family_name,
+                self._where_clause()
+            )
+            execute(qs, self._where_values(), self._consistency)
+
 
 class DMLQuery(object):
     """
@@ -749,13 +834,16 @@ class DMLQuery(object):
 
     unlike the read query object, this is mutable
     """
+    _ttl = None
+    _consistency = None
 
-    def __init__(self, model, instance=None, batch=None):
+    def __init__(self, model, instance=None, batch=None, ttl=None, consistency=None):
         self.model = model
         self.column_family_name = self.model.column_family_name()
         self.instance = instance
         self._batch = batch
-        pass
+        self._ttl = ttl
+        self._consistency = consistency
 
     def batch(self, batch_obj):
         if batch_obj is not None and not isinstance(batch_obj, BatchQuery):
@@ -763,84 +851,11 @@ class DMLQuery(object):
         self._batch = batch_obj
         return self
 
-    def save(self):
+    def _delete_null_columns(self):
         """
-        Creates / updates a row.
-        This is a blind insert call.
-        All validation and cleaning needs to happen
-        prior to calling this.
+        executes a delete query to remove columns that have changed to null
         """
-        if self.instance is None:
-            raise CQLEngineException("DML Query intance attribute is None")
-        assert type(self.instance) == self.model
-
-        #organize data
-        value_pairs = []
-        values = self.instance._as_dict()
-
-        #get defined fields and their column names
-        for name, col in self.model._columns.items():
-            val = values.get(name)
-            if val is None: continue
-            value_pairs += [(col.db_field_name, val)]
-
-        #construct query string
-        field_names = zip(*value_pairs)[0]
-        field_ids = {n:uuid4().hex for n in field_names}
-        field_values = dict(value_pairs)
-        query_values = {field_ids[n]:field_values[n] for n in field_names}
-
-        qs = []
-        if self.instance._has_counter or self.instance._can_update():
-            qs += ["UPDATE {}".format(self.column_family_name)]
-            qs += ["SET"]
-
-            set_statements = []
-            #get defined fields and their column names
-            for name, col in self.model._columns.items():
-                if not col.is_primary_key:
-                    val = values.get(name)
-                    if val is None:
-                        continue
-                    if isinstance(col, (BaseContainerColumn, Counter)):
-                        #remove value from query values, the column will handle it
-                        query_values.pop(field_ids.get(name), None)
-
-                        val_mgr = self.instance._values[name]
-                        set_statements += col.get_update_statement(val, val_mgr.previous_value, query_values)
-
-                    else:
-                        set_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
-            qs += [', '.join(set_statements)]
-
-            qs += ['WHERE']
-
-            where_statements = []
-            for name, col in self.model._primary_keys.items():
-                where_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
-
-            qs += [' AND '.join(where_statements)]
-
-            # clear the qs if there are no set statements and this is not a counter model
-            if not set_statements and not self.instance._has_counter:
-                qs = []
-
-        else:
-            qs += ["INSERT INTO {}".format(self.column_family_name)]
-            qs += ["({})".format(', '.join(['"{}"'.format(f) for f in field_names]))]
-            qs += ['VALUES']
-            qs += ["({})".format(', '.join([':'+field_ids[f] for f in field_names]))]
-
-        qs = ' '.join(qs)
-
-        # skip query execution if it's empty
-        # caused by pointless update queries
-        if qs:
-            if self._batch:
-                self._batch.add_query(qs, query_values)
-            else:
-                execute(qs, query_values)
-
+        values, field_names, field_ids, field_values, query_values = self._get_query_values()
 
         # delete nulled columns and removed map keys
         qs = ['DELETE']
@@ -874,6 +889,136 @@ class DMLQuery(object):
             else:
                 execute(qs, query_values)
 
+    def update(self):
+        """
+        updates a row.
+        This is a blind update call.
+        All validation and cleaning needs to happen
+        prior to calling this.
+        """
+        if self.instance is None:
+            raise CQLEngineException("DML Query intance attribute is None")
+        assert type(self.instance) == self.model
+
+        values, field_names, field_ids, field_values, query_values = self._get_query_values()
+
+        qs = []
+        qs += ["UPDATE {}".format(self.column_family_name)]
+        qs += ["SET"]
+
+        set_statements = []
+        #get defined fields and their column names
+        for name, col in self.model._columns.items():
+            if not col.is_primary_key:
+                val = values.get(name)
+
+                # don't update something that is null
+                if val is None:
+                    continue
+
+                # don't update something if it hasn't changed
+                if not self.instance._values[name].changed and not isinstance(col, Counter):
+                    continue
+
+                # add the update statements
+                if isinstance(col, (BaseContainerColumn, Counter)):
+                    #remove value from query values, the column will handle it
+                    query_values.pop(field_ids.get(name), None)
+
+                    val_mgr = self.instance._values[name]
+                    set_statements += col.get_update_statement(val, val_mgr.previous_value, query_values)
+
+                else:
+                    set_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
+        qs += [', '.join(set_statements)]
+
+        qs += ['WHERE']
+
+        where_statements = []
+        for name, col in self.model._primary_keys.items():
+            where_statements += ['"{}" = :{}'.format(col.db_field_name, field_ids[col.db_field_name])]
+
+        qs += [' AND '.join(where_statements)]
+
+        if self._ttl:
+            qs += ["USING TTL {}".format(self._ttl)]
+
+        # clear the qs if there are no set statements and this is not a counter model
+        if not set_statements and not self.instance._has_counter:
+            qs = []
+
+        qs = ' '.join(qs)
+        # skip query execution if it's empty
+        # caused by pointless update queries
+        if qs:
+            if self._batch:
+                self._batch.add_query(qs, query_values)
+            else:
+                execute(qs, query_values, consistency_level=self._consistency)
+
+        self._delete_null_columns()
+
+    def _get_query_values(self):
+        """
+        returns all the data needed to do queries
+        """
+        #organize data
+        value_pairs = []
+        values = self.instance._as_dict()
+
+        #get defined fields and their column names
+        for name, col in self.model._columns.items():
+            val = values.get(name)
+            if col._val_is_null(val): continue
+            value_pairs += [(col.db_field_name, val)]
+
+        #construct query string
+        field_names = zip(*value_pairs)[0]
+        field_ids = {n:uuid4().hex for n in field_names}
+        field_values = dict(value_pairs)
+        query_values = {field_ids[n]:field_values[n] for n in field_names}
+        return values, field_names, field_ids, field_values, query_values
+
+    def save(self):
+        """
+        Creates / updates a row.
+        This is a blind insert call.
+        All validation and cleaning needs to happen
+        prior to calling this.
+        """
+        if self.instance is None:
+            raise CQLEngineException("DML Query intance attribute is None")
+        assert type(self.instance) == self.model
+
+        values, field_names, field_ids, field_values, query_values = self._get_query_values()
+
+        qs = []
+        if self.instance._has_counter or self.instance._can_update():
+            return self.update()
+        else:
+            qs += ["INSERT INTO {}".format(self.column_family_name)]
+            qs += ["({})".format(', '.join(['"{}"'.format(f) for f in field_names]))]
+            qs += ['VALUES']
+            qs += ["({})".format(', '.join([':'+field_ids[f] for f in field_names]))]
+
+        if self._ttl:
+            qs += ["USING TTL {}".format(self._ttl)]
+
+        qs += []
+        qs = ' '.join(qs)
+
+
+        # skip query execution if it's empty
+        # caused by pointless update queries
+        if qs:
+            if self._batch:
+                self._batch.add_query(qs, query_values)
+            else:
+                execute(qs, query_values, self._consistency)
+
+        # delete any nulled columns
+        self._delete_null_columns()
+
     def delete(self):
         """ Deletes one instance """
         if self.instance is None:
@@ -893,6 +1038,6 @@ class DMLQuery(object):
         if self._batch:
             self._batch.add_query(qs, field_values)
         else:
-            execute(qs, field_values)
+            execute(qs, field_values, self._consistency)
 
 
